@@ -1,33 +1,9 @@
 #
-#  Copyright (c) 1997-2002 The Protein Laboratory, University of Copenhagen
-#  All rights reserved.
-#
-#  Redistribution and use in source and binary forms, with or without
-#  modification, are permitted provided that the following conditions
-#  are met:
-#  1. Redistributions of source code must retain the above copyright
-#     notice, this list of conditions and the following disclaimer.
-#  2. Redistributions in binary form must reproduce the above copyright
-#     notice, this list of conditions and the following disclaimer in the
-#     documentation and/or other materials provided with the distribution.
-#
-#  THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
-#  ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
-#  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
-#  ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
-#  FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
-#  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
-#  OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
-#  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
-#  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
-#  OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
-#  SUCH DAMAGE.
-#
 #  Created by:
 #     Anton Berezin  <tobez@tobez.org>
 #     Dmitry Karasik <dk@plab.ku.dk> 
 #
-#  $Id$
+package Prima::Classes;
 use Prima;
 use Prima::Const;
 
@@ -35,6 +11,7 @@ use Prima::Const;
 package Prima::Object;
 use vars qw(@hooks);
 use strict;
+use warnings;
 use Carp;
 
 sub CLONE_SKIP { 1 }
@@ -327,8 +304,7 @@ package Prima::Drawable;
 use vars qw(@ISA);
 @ISA = qw(Prima::Component);
 use Prima::Bidi qw(is_bidi);
-
-my $sysData = Prima::Application-> get_system_info;
+use Prima::Drawable::Basic;
 
 sub profile_default
 {
@@ -394,188 +370,13 @@ sub put_image
 		defined ($_[4]) ? $_[4] : $_[0]-> rop
 	) if $_[3]
 }
+
 sub stretch_image { 
 	$_[0]-> put_image_indirect( 
 		@_[5,1,2], 0, 0, 
 		@_[3,4], $_[5]-> size, 
 		defined ($_[6]) ? $_[6] : $_[0]-> rop
 	) if $_[5]
-} 
-
-sub rect3d
-{
-	my ( $self, $x, $y, $x1, $y1, $width, $lColor, $rColor, $backColor) = @_;
-	my $c = $self-> color;
-	if ( defined $backColor)
-	{
-		if ( $backColor == cl::Back) {
-			$self-> clear( $x + $width, $y + $width, $x1 - $width, $y1 - $width);
-		} else {
-			$self-> color( $backColor);
-			$self-> bar( $x + $width, $y + $width, $x1 - $width, $y1 - $width);
-		}
-	}
-	$lColor = $rColor = cl::Black if $self-> get_bpp == 1;
-	$self-> color( $c), return if $width <= 0;
-	$self-> color( $lColor);
-	$width = ( $y1 - $y) / 2 if $width > ( $y1 - $y) / 2;
-	$width = ( $x1 - $x) / 2 if $width > ( $x1 - $x) / 2;
-	$self-> lineWidth( 0);
-	my $i;
-	for ( $i = 0; $i < $width; $i++) {
-		$self-> line( $x + $i, $y + $i, $x + $i, $y1 - $i);
-		$self-> line( $x + $i + 1, $y1 - $i, $x1 - $i, $y1 - $i);
-	}
-	$self-> color( $rColor);
-	for ( $i = 0; $i < $width; $i++) {
-		$self-> line( $x1 - $i, $y + $i, $x1 - $i, $y1 - $i);
-		$self-> line( $x + $i + 1, $y + $i, $x1 - $i, $y + $i);
-	}
-	$self-> color( $c);
-}
-
-sub rect_focus
-{
-	my ( $canvas, $x, $y, $x1, $y1, $width) = @_;
-	( $x, $x1) = ( $x1, $x) if $x > $x1;
-	( $y, $y1) = ( $y1, $y) if $y > $y1;
-
-	$width = 1 if !defined $width || $width < 1;
-	my ( $cl, $cl2, $rop) = ( $canvas-> color, $canvas-> backColor, $canvas-> rop);
-	my $fp = $canvas-> fillPattern;
-	$canvas-> set(
-		fillPattern => fp::SimpleDots,
-		color       => cl::Set,
-		backColor   => cl::Clear,
-		rop         => rop::XorPut,
-	);
-
-	if ( $width * 2 >= $x1 - $x or $width * 2 >= $y1 - $y) {
-		$canvas-> bar( $x, $y, $x1, $y1);
-	} else {
-		$width -= 1;
-		$canvas-> bar( $x, $y, $x1, $y + $width);
-		$canvas-> bar( $x, $y1 - $width, $x1, $y1);
-		$canvas-> bar( $x, $y + $width + 1, $x + $width, $y1 - $width - 1);
-		$canvas-> bar( $x1 - $width, $y + $width + 1, $x1, $y1 - $width - 1);
-	}
-
-	$canvas-> set(
-		fillPattern => $fp,
-		backColor   => $cl2,
-		color       => $cl,
-		rop         => $rop,
-	);
-}
-
-sub draw_text
-{
-	my ( $canvas, $string, $x, $y, $x2, $y2, $flags, $tabIndent) = @_;
-
-	$flags     = dt::Default unless defined $flags;
-	$tabIndent = 1 if !defined( $tabIndent) || $tabIndent < 0;
-
-	$x2 = int( $x2);
-	$x  = int( $x);
-	$y2 = int( $y2);
-	$y  = int( $y);
-
-	my ( $w, $h) = ( $x2 - $x + 1, $y2 - $y + 1);
-
-	return 0 if $w <= 0 || $h <= 0;
-
-	my $twFlags = tw::ReturnLines |
-		(( $flags & dt::DrawMnemonic  ) ? ( tw::CalcMnemonic | tw::CollapseTilde) : 0) |
-		(( $flags & dt::DrawSingleChar) ? 0 : tw::BreakSingle ) |
-		(( $flags & dt::NewLineBreak  ) ? tw::NewLineBreak : 0) |
-		(( $flags & dt::SpaceBreak    ) ? tw::SpaceBreak   : 0) |
-		(( $flags & dt::WordBreak     ) ? tw::WordBreak    : 0) |
-		(( $flags & dt::ExpandTabs    ) ? ( tw::ExpandTabs | tw::CalcTabs) : 0)
-	;
-
-	my @lines = @{$canvas-> text_wrap( $string, 
-		( $flags & dt::NoWordWrap) ? 1000000 : $w, 
-		$twFlags, $tabIndent
-	)};
-
-	my $tildes;
-	$tildes = pop @lines if $flags & dt::DrawMnemonic;
-
-	return 0 unless scalar @lines;
-
-	if (($flags & dt::BidiText) && $Prima::Bidi::available) {
-		$_ = Prima::Bidi::visual($_) for @lines;
-	}
-
-	my @clipSave;
-	my $fh = $canvas-> font-> height +
-		(( $flags & dt::UseExternalLeading) ? 
-			$canvas-> font-> externalLeading : 
-			0
-		);
-	my ( $linesToDraw, $retVal);
-	my $valign = $flags & 0xC;
-
-	if ( $flags & dt::QueryHeight) {
-		$linesToDraw = scalar @lines;
-		$h = $retVal = $linesToDraw * $fh;
-	} else {
-		$linesToDraw = int( $retVal = ( $h / $fh));
-		$linesToDraw++ 
-			if (( $h % $fh) > 0) and ( $flags & dt::DrawPartial);
-		$valign      = dt::Top 
-			if $linesToDraw < scalar @lines;
-		$linesToDraw = $retVal = scalar @lines 
-			if $linesToDraw > scalar @lines;
-	}
-
-	if ( $flags & dt::UseClip) {
-		@clipSave = $canvas-> clipRect;
-		$canvas-> clipRect( $x, $y, $x + $w, $y + $h);
-	}
-
-	if ( $valign == dt::Top) {
-		$y = $y2;
-	} elsif ( $valign == dt::VCenter) {
-		$y = $y2 - int(( $h - $linesToDraw * $fh) / 2);
-	} else {
-		$y += $linesToDraw * $fh;
-	}
-
-	my ( $starty, $align) = ( $y, $flags & 0x3);
-
-	for ( @lines) {
-		last unless $linesToDraw--;
-		my $xx;
-		if ( $align == dt::Left) {
-			$xx = $x;
-		} elsif ( $align == dt::Center) {
-			$xx = $x + int(( $w - $canvas-> get_text_width( $_)) / 2);
-		} else {
-			$xx = $x2 - $canvas-> get_text_width( $_);
-		}
-		$y -= $fh;
-		$canvas-> text_out( $_, $xx, $y);
-	}
-
-	if (( $flags & dt::DrawMnemonic) and ( defined $tildes-> {tildeLine})) {
-		my $tl = $tildes-> {tildeLine};
-		my $xx = $x;
-		if ( $align == dt::Center) {
-			$xx = $x + int(( $w - $canvas-> get_text_width( $lines[ $tl])) / 2);
-		} elsif ( $align == dt::Right) {
-			$xx = $x2 - $canvas-> get_text_width( $lines[ $tl]);
-		}
-		$tl++;
-		$canvas-> line( 
-			$xx + $tildes-> {tildeStart}, $starty - $fh * $tl,
-			$xx + $tildes-> {tildeEnd}  , $starty - $fh * $tl
-		);
-	}
-
-	$canvas-> clipRect( @clipSave) if $flags & dt::UseClip;
-
-	return $retVal;
 }
 
 sub text_out_bidi
@@ -609,7 +410,7 @@ sub profile_default
 		conversion    => ict::Optimized,
 		data          => '',
 		height        => 0,
-		hScaling      => 1,
+		scaling       => ist::Box,
 		palette       => [0, 0, 0, 0xFF, 0xFF, 0xFF],
 		colormap      => undef,
 		preserveType  => 0,
@@ -618,7 +419,6 @@ sub profile_default
 		resolution    => [0, 0],
 		type          => im::Mono,
 		width         => 0,
-		vScaling      => 1,
 	);
 	@$def{keys %prf} = values %prf;
 	return $def;
@@ -1273,12 +1073,17 @@ sub rect_bevel
 		push @c3d, $back, 0x404040;
 	}
 
+	if ( my $g = $opt{gradient} ) {
+		for (@{$g->{palette} // []}) {
+			$_ = $self->map_color($_) if $_ & cl::SysFlag;
+		}
+	}
+
 	my $hw = int( $width / 2);
-	$canvas-> rect3d( $x, $y, $x1, $y1, $hw, @c3d[2,3], $fill);
+	$canvas-> rect3d( $x, $y, $x1, $y1, $hw, @c3d[2,3], $opt{gradient} // $fill);
 	$canvas-> rect3d( $x + $hw, $y + $hw, $x1 - $hw, $y1 - $hw, $width - $hw, @c3d[0,1]);
 }
 
-
 # class Window
 package Prima::Window;
 use vars qw(@ISA);
