@@ -1,33 +1,3 @@
-#
-#  Copyright (c) 1997-2002 The Protein Laboratory, University of Copenhagen
-#  All rights reserved.
-#
-#  Redistribution and use in source and binary forms, with or without
-#  modification, are permitted provided that the following conditions
-#  are met:
-#  1. Redistributions of source code must retain the above copyright
-#     notice, this list of conditions and the following disclaimer.
-#  2. Redistributions in binary form must reproduce the above copyright
-#     notice, this list of conditions and the following disclaimer in the
-#     documentation and/or other materials provided with the distribution.
-#
-#  THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
-#  ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
-#  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
-#  ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
-#  FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
-#  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
-#  OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
-#  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
-#  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
-#  OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
-#  SUCH DAMAGE.
-#
-#  Created by:
-#     Anton Berezin  <tobez@plab.ku.dk>
-#     Dmitry Karasik <dk@plab.ku.dk> 
-#
-#  $Id$
 package Prima::Const;
 use Prima '';
 use Carp;
@@ -40,6 +10,9 @@ sub AUTOLOAD {
 	goto &$AUTOLOAD;
 }
 
+use strict;
+use warnings;
+
 # double lines for CPAN indexer, we don't want these packages in top-level namespace
 package 
     nt; *AUTOLOAD =  \&Prima::Const::AUTOLOAD;	# notification types
@@ -89,6 +62,8 @@ package
     ict; *AUTOLOAD = \&Prima::Const::AUTOLOAD;	# Image conversion types
 package 
     is; *AUTOLOAD =  \&Prima::Const::AUTOLOAD;	# Image statistics types
+package 
+    ist; *AUTOLOAD = \&Prima::Const::AUTOLOAD;	# Image scaling types
 package 
     am; *AUTOLOAD =  \&Prima::Const::AUTOLOAD;	# Icon auto masking
 package 
@@ -117,6 +92,8 @@ package
     gt; *AUTOLOAD = \&Prima::Const::AUTOLOAD;	# geometry manager types
 package 
     ps; *AUTOLOAD = \&Prima::Const::AUTOLOAD;	# paint states
+package 
+    scr; *AUTOLOAD = \&Prima::Const::AUTOLOAD;	# scroll() results
 
 1;
 
@@ -158,7 +135,6 @@ See also L<Prima::Image/autoMasking>
 
 See L<Prima::Application/get_system_info>
 
-	apc::Os2    
 	apc::Win32  
 	apc::Unix   
 
@@ -367,6 +343,7 @@ See L<Prima::Utils/query_drive_type>
 				of the lines drawn
 	dt::NoWordWrap        - performs no word wrapping by the width of the boundaries
 	dt::WordWrap          - performs word wrapping by the width of the boundaries 
+	dt::BidiText          - use bidirectional formatting, if available
 	dt::Default           - dt::NewLineBreak|dt::WordBreak|dt::ExpandTabs|
 				dt::UseExternalLeading
 
