@@ -1,6 +1,7 @@
 package Prima::DetailedOutline;
 
 use strict;
+use warnings;
 use Prima::Outlines;
 use Prima::DetailedList;
 
@@ -83,6 +84,7 @@ sub init {
 	$self->$_( $profile{$_}) for qw(autoRecalc columns mainColumn);
 	$self->autowidths unless scalar @{$profile{widths}};
 	$self->{recalc} = 1 if $profile{autoRecalc};
+	$self->{align} = ta::Left;
 	return %profile;
 }
 
@@ -135,11 +137,10 @@ sub draw_items
 	for (@widths) { $extent += $_ }
 	$canvas->backColor($clrs[3]);
 	for ( $i = 0; $i < $icount; $i++) {
-		my ($node, $x, $y, $x2, $y2, $position, $focused) = @{$$paintStruc[$i]};
+		my ($node, $x, $y, $x2, $y2, $position, $selected, $focused, $prelight) = @{$$paintStruc[$i]};
+		next unless $prelight || $selected;
 		$x = $xend + 1 if ($xend < $x);
-		if ($focused) {
-			$canvas->clear($x, $y, $extent, $y2);
-		}
+		$self->draw_item_background( $canvas, $x, $y, $extent, $y2, $prelight, $selected ? $clrs[3] : $clrs[1]);
 	}
 	$canvas->backColor($clrs[0]);
 
