@@ -1,33 +1,5 @@
-# $Id$
-#
-#  Copyright (c) 1997-2004 Dmitry Karasik
-#  All rights reserved.
-#
-#  Redistribution and use in source and binary forms, with or without
-#  modification, are permitted provided that the following conditions
-#  are met:
-#  1. Redistributions of source code must retain the above copyright
-#     notice, this list of conditions and the following disclaimer.
-#  2. Redistributions in binary form must reproduce the above copyright
-#     notice, this list of conditions and the following disclaimer in the
-#     documentation and/or other materials provided with the distribution.
-#
-#  THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
-#  ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
-#  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
-#  ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
-#  FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
-#  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
-#  OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
-#  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
-#  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
-#  OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
-#  SUCH DAMAGE.
-#
-#  Created by Dmitry Karasik <dmitry@karasik.eu.org>
-#
-#  $Id$
 use strict;
+use warnings;
 
 package Prima::EventHook;
 use vars qw($hook $auto_hook %hooks %groups);
@@ -77,7 +49,7 @@ sub install
 		}
 	}
 
-	Prima::Component-> event_hook( $hook = \&hook_proc) 
+	Prima::Component-> event_hook( $hook = \&_hook_proc) 
 		if $auto_hook && !$hook;
 }
 
@@ -93,7 +65,7 @@ sub deinstall
 		if !$total && $hook && $auto_hook;
 }
 
-sub hook_proc
+sub _hook_proc
 {
 	my ( $object, $event, @params) = @_;
 	for ( '*', $event) {
