@@ -1,35 +1,8 @@
-#
-#  Copyright (c) 1997-2002 The Protein Laboratory, University of Copenhagen
-#  All rights reserved.
-#
-#  Redistribution and use in source and binary forms, with or without
-#  modification, are permitted provided that the following conditions
-#  are met:
-#  1. Redistributions of source code must retain the above copyright
-#     notice, this list of conditions and the following disclaimer.
-#  2. Redistributions in binary form must reproduce the above copyright
-#     notice, this list of conditions and the following disclaimer in the
-#     documentation and/or other materials provided with the distribution.
-#
-#  THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
-#  ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
-#  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
-#  ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
-#  FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
-#  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
-#  OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
-#  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
-#  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
-#  OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
-#  SUCH DAMAGE.
-#
 #  Created by:
 #     Anton Berezin  <tobez@tobez.org>
 #     Dmitry Karasik <dk@plab.ku.dk> 
-#
-#  $Id$
-#
 use strict;
+use warnings;
 use Prima::ScrollWidget;
 
 package Prima::ImageViewer;
@@ -477,12 +450,12 @@ sub stretch      {($#_)?$_[0]-> set_stretch($_[1]):return $_[0]-> {stretch}}
 
 sub PreviewImage_HeaderReady
 { 
-	my ( $self, $image) = @_;
+	my ( $self, $image, $extras) = @_;
 	my $db;
 	eval {
 		$db = Prima::DeviceBitmap-> new(
-			width    => $image-> width,
-			height   => $image-> height,
+			width    => $image->width,
+			height   => $image->height,
 		);
 	};
 	return unless $db;
@@ -490,16 +463,16 @@ sub PreviewImage_HeaderReady
 	$self-> image($db);
         $self-> image-> backColor(0);
         $self-> image-> clear;
-	$self-> {__preview_image} = 1;
+	$self-> {__watch}->{preview} = 1;
 }
 
 sub PreviewImage_DataReady
 { 
 	my ( $self, $image, $x, $y, $w, $h) = @_;
-	return unless $self-> {__preview_image};
+	return unless exists $self-> {__watch} && $self->{__watch}->{preview};
 
 	# do not update if DataReady covers the whole image at once
-	return if $y == 0 and $h == $image-> height;
+	return if $y == 0 and $x == 0 && $h == $image-> height && $w == $image-> width;
 
 	$self-> image-> put_image_indirect( $image, $x, $y, $x, $y, $w, $h, $w, $h, rop::CopyPut);
 	my @r = $self-> point2screen( $x, $y, $x + $w, $y + $h);
@@ -520,26 +493,27 @@ sub watch_load_progress
 		$image-> add_notification( 'HeaderReady', \&PreviewImage_HeaderReady, $self),
 		$image-> add_notification( 'DataReady',   \&PreviewImage_DataReady,   $self)
 		;
-	$self-> {__watch_notifications} = [ @ids ];
+	$self-> {__watch} = {
+		notifications => \@ids,
+	};
 }
 
 sub unwatch_load_progress
 {
 	my ( $self, $clear_image) = @_;
 
-	return unless $self-> {__watch_notifications};
+	return unless $self-> {__watch};
 
 	if ( $self-> {image}) {
 		$self-> {image}-> remove_notification($_) 
-			for @{ $self-> {__watch_notifications} };
+			for @{ $self-> {__watch}-> {notifications} };
 	}
-	delete $self-> {__watch_notifications};
 
-	$clear_image = 1 unless defined $clear_image;
-	if ( $self-> {__preview_image}) {
-		$self-> image( undef) if $clear_image;
-		delete $self-> {__preview_image};
+	if ( $self-> {__watch}-> {preview} && ($clear_image // 1)) {
+		$self-> image( undef);
 	}
+	
+	delete $self-> {__watch};
 }
 
 1;
