@@ -1,34 +1,7 @@
-#
-#  Copyright (c) 1997-2002 The Protein Laboratory, University of Copenhagen
-#  All rights reserved.
-#
-#  Redistribution and use in source and binary forms, with or without
-#  modification, are permitted provided that the following conditions
-#  are met:
-#  1. Redistributions of source code must retain the above copyright
-#     notice, this list of conditions and the following disclaimer.
-#  2. Redistributions in binary form must reproduce the above copyright
-#     notice, this list of conditions and the following disclaimer in the
-#     documentation and/or other materials provided with the distribution.
-#
-#  THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
-#  ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
-#  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
-#  ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
-#  FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
-#  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
-#  OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
-#  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
-#  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
-#  OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
-#  SUCH DAMAGE.
-#
-#  Created by Dmitry Karasik <dk@plab.ku.dk>
-#
-#  $Id: MsgBox.pm,v 1.21 2007/05/18 12:21:06 dk Exp $
 package Prima::MsgBox;
 
 use strict;
+use warnings;
 require Exporter;
 use vars qw(@ISA @EXPORT @EXPORT_OK);
 @ISA = qw(Exporter);
@@ -106,7 +79,13 @@ sub insert_buttons
 
 sub message_box
 {
-	my ( $title, $text, $options, $extras) = @_;
+	my ( $title, $text, $options, @extras) = @_;
+	
+	my $extras = 
+		( 1 == @extras and (ref($extras[0])||'') eq 'HASH') ?
+	   	$extras[0] : # old style
+		{ @extras }; # new style
+
 	$options = mb::Ok | mb::Error unless defined $options;
 	$options |= mb::Ok unless $options & 0x0FF;
 	my $buttons = $options & 0xFF;
@@ -133,6 +112,7 @@ sub message_box
 		visible       => 0,
 		text          => $title,
 		font          => $::application-> get_message_font,
+		owner         => $extras-> {owner} || $::main_window || $::application,
 		onExecute     => sub {
 			Prima::Utils::beep( $options) if $options && !$nosound;
 		},
@@ -209,8 +189,13 @@ sub message
 
 sub input_box
 {
-	my ( $title, $text, $string, $buttons, $extras) = @_;
+	my ( $title, $text, $string, $buttons, @extras) = @_;
 	$buttons = mb::Ok|mb::Cancel unless defined $buttons;
+	
+	my $extras = 
+		( 1 == @extras and (ref($extras[0])||'') eq 'HASH') ?
+	   	$extras[0] : # old style
+		{ @extras }; # new style
 
 	my $dlg = Prima::Dialog-> create(
 		centered      => 1,
@@ -289,7 +274,7 @@ dialog boxes.
 
 =over
 
-=item input_box TITLE, LABEL, INPUT_STRING, [ BUTTONS = mb::OkCancel, PROFILES = {} ]
+=item input_box TITLE, LABEL, INPUT_STRING, [ BUTTONS = mb::OkCancel, %PROFILES ]
 
 Invokes standard dialog box, that contains an input line, a text label, and
 buttons used for ending dialog session. The dialog box uses TITLE string to
@@ -310,11 +295,11 @@ text entered. The input text is not restored to its original value if the
 dialog was cancelled. In scalar context returns the text entered, if the dialog
 was ended with C<mb::OK> or C<mb::Yes> result, or C<undef> otherwise.
 
-=item message TEXT, [ OPTIONS = mb::Ok | mb::Error, PROFILES = {} ]
+=item message TEXT, [ OPTIONS = mb::Ok | mb::Error, %PROFILES ]
 
 Same as C<message_box> call, with application name passed as the title string.
 
-=item message_box TITLE, TEXT, [ OPTIONS = mb::Ok | mb::Error, PROFILES = {} ] 
+=item message_box TITLE, TEXT, [ OPTIONS = mb::Ok | mb::Error, %PROFILES ]
 
 Invokes standard dialog box, that contains a text label, a predefined icon, and
 buttons used for ending dialog session. The dialog box uses TITLE string to
@@ -390,19 +375,25 @@ can be set with the hash reference under C<buttons> key.  The hash is a
 profile, passed to the button as creation parameters. For example, to change
 the text and behavior of a button, the following construct can be used:
 
-	Prima::MsgBox::message( 'Hello', mb::OkCancel, {
+	Prima::MsgBox::message( 'Hello', mb::OkCancel,
 		buttons => {
 			mb::Ok, {
 				text     => '~Hello',
 				onClick  => sub { Prima::message('Hello indeed!'); }
 			}
 		}
-	});
+	);
 
 If it is not desired that the dialog must be closed when the user presses a
 button, its C<::modalResult> property ( see L<Prima::Buttons> ) must be reset
 to 0.
 
+=item owner WINDOW
+
+If set, the dialog owner is set to WINDOW, otherwise to C<$::main_window>.
+Necessary to maintain window stack order under some window managers, to disallow
+windows to be brought over the message box.
+
 =back
 
 =head1 AUTHOR
