@@ -1,32 +1,7 @@
-#
-#  Copyright (c) 1997-2002 The Protein Laboratory, University of Copenhagen
-#  All rights reserved.
-#
-#  Redistribution and use in source and binary forms, with or without
-#  modification, are permitted provided that the following conditions
-#  are met:
-#  1. Redistributions of source code must retain the above copyright
-#     notice, this list of conditions and the following disclaimer.
-#  2. Redistributions in binary form must reproduce the above copyright
-#     notice, this list of conditions and the following disclaimer in the
-#     documentation and/or other materials provided with the distribution.
-#
-#  THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
-#  ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
-#  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
-#  ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
-#  FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
-#  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
-#  OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
-#  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
-#  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
-#  OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
-#  SUCH DAMAGE.
-#
-#  Created by Dmitry Karasik <dk@plab.ku.dk>
-#
-#  $Id$
+package Prima::Notebooks;
+
 use strict;
+use warnings;
 use Prima::Const;
 use Prima::Classes;
 use Prima::IntUtils;
@@ -149,33 +124,17 @@ sub recalc_widths
 	$self-> {widths}    = [@w];
 }
 
-sub on_mousedown
+sub x2pos
 {
-	my ( $self, $btn, $mod, $x, $y) = @_;
-	return if $self-> {mouseTransaction};
-	$self-> clear_event;
+	my ( $self, $x ) = @_;
+
 	my ( $a, $ww, $ft, $lt) = (
 		$self-> {arrows}, $self-> {widths}, $self-> {firstTab}, $self-> {lastTab}
 	);
-
-	if (( $a & 1) and ( $x < DefLeftX + DefGapX * 2 + DefArrowX)) {
-		$self-> firstTab( $self-> firstTab - 1);
-		$self-> capture(1);
-		$self-> {mouseTransaction} = -1;
-		$self-> scroll_timer_start;
-		$self-> scroll_timer_semaphore(0);
-		return;
-	}
+	return -1 if ( $a & 1) and ( $x < DefLeftX + DefGapX * 2 + DefArrowX);
 
 	my @size = $self-> size;
-	if (( $a & 2) and ( $x >= $size[0] - DefLeftX - DefGapX * 2 - DefArrowX)) {
-		$self-> firstTab( $self-> firstTab + 1);
-		$self-> capture(1);
-		$self-> {mouseTransaction} = 1;
-		$self-> scroll_timer_start;
-		$self-> scroll_timer_semaphore(0);
-		return;
-	}
+	return -2 if ( $a & 2) and ( $x >= $size[0] - DefLeftX - DefGapX * 2 - DefArrowX);
 
 	my $w = DefLeftX;
 	$w += DefGapX + DefArrowX if $a & 1;
@@ -185,13 +144,41 @@ sub on_mousedown
 		$found = $i, last if $x < $w + $$ww[$i] + DefGapX;
 		$w += $$ww[$i] + DefGapX;
 	}
-	return unless defined $found;
-	
-	if ( $found == $self-> {tabIndex}) {
-		$self-> focusedTab( $found);
+	return $found;
+}
+
+sub on_mousedown
+{
+	my ( $self, $btn, $mod, $x, $y) = @_;
+	return if $self-> {mouseTransaction};
+	$self-> clear_event;
+
+	my $pos = $self-> x2pos($x);
+	return unless defined $pos;
+
+	if ($pos == -1) {
+		$self-> firstTab( $self-> firstTab - 1);
+		$self-> capture(1);
+		$self-> {mouseTransaction} = -1;
+		$self-> scroll_timer_start;
+		$self-> scroll_timer_semaphore(0);
+		return;
+	}
+
+	if ($pos == -2) {
+		$self-> firstTab( $self-> firstTab + 1);
+		$self-> capture(1);
+		$self-> {mouseTransaction} = 1;
+		$self-> scroll_timer_start;
+		$self-> scroll_timer_semaphore(0);
+		return;
+	}
+
+	if ( $pos == $self-> {tabIndex}) {
+		$self-> focusedTab( $pos);
 		$self-> focus;
 	} else {
-		$self-> tabIndex( $found);
+		$self-> tabIndex( $pos);
 	}
 }
 
@@ -216,7 +203,16 @@ sub on_mouseup
 sub on_mousemove
 {
 	my ( $self, $mod, $x, $y) = @_;
-	return unless $self-> {mouseTransaction};
+	unless ($self-> {mouseTransaction}) {
+		if ( $self-> enabled ) {
+			my $prelight = $self-> x2pos($x);
+			if (( $prelight // '') ne ($self->{prelight} // '')) {
+				$self->{prelight} = $prelight;
+				$self-> repaint;
+			}
+		}
+		return;
+	}
 	return unless $self-> scroll_timer_semaphore;
 
 	$self-> scroll_timer_semaphore(0);
@@ -225,6 +221,12 @@ sub on_mousemove
 	$self-> notify(q(MouseUp),1,0,0,0) if $ft == $self-> firstTab;
 }
 
+sub on_mouseleave
+{
+	my $self = shift;
+	$self-> repaint if defined( delete $self->{prelight} );
+}
+
 sub on_mouseclick
 {
 	my $self = shift;
@@ -265,8 +267,12 @@ sub on_keydown
 sub on_paint
 {
 	my ($self,$canvas) = @_;
-	my @clr  = ( $self-> color, $self-> backColor);
-	@clr = ( $self-> disabledColor, $self-> disabledBackColor) if ( !$self-> enabled);
+	my @clr;
+	if ( $self-> enabled ) {
+		@clr = ( $self-> color, $self-> backColor);
+	} else {
+		@clr = ( $self-> disabledColor, $self-> disabledBackColor);
+	}
 	my @c3d  = ( $self-> dark3DColor, $self-> light3DColor);
 	my @size = $canvas-> size;
 
@@ -421,10 +427,11 @@ sub on_measuretab
 sub on_drawtab
 {
 	my ( $self, $canvas, $i, $clr, $poly, $poly2) = @_;
-	
-	$canvas-> color(( $self-> {colored} && ( $i >= 0)) ?
-		( $warpColors[ $i % scalar @warpColors]) : $$clr[1]);
-		
+
+	my $color = ( $self-> {colored} && ( $i >= 0)) ?
+		( $warpColors[ $i % scalar @warpColors]) : $$clr[1];
+	$color = $self-> prelight_color($color) if ($self->{prelight} // '') eq ($i // '');
+	$canvas-> color($color);
 	$canvas-> fillpoly( $poly);
 	$canvas-> fillpoly( $poly2) if $poly2;
 	$canvas-> color( $$clr[3]);
@@ -1115,8 +1122,33 @@ sub on_paint
 		);
 
 		my $y = $size[1] - DefBorderX + Prima::TabSet::DefGapY;
-		my $x  = $size[0] - DefBorderX - DefBookmarkX;
+		my $x = $size[0] - DefBorderX - DefBookmarkX;
 		return if $y < DefBorderX * 2 + DefBookmarkX;
+		
+		my $a  = 0;
+		my ($pi, $mpi) = ( 
+			$self-> {notebook}-> pageIndex, 
+			$self-> {notebook}-> pageCount - 1
+		);
+		$a |= 1 if $pi > 0;
+		$a |= 2 if $pi < $mpi;
+
+		if ( my $p = $self->{prelight}) {
+			$canvas-> color( $self-> prelight_color($c3d[1]));
+			if ( $p < 0 && $a & 1) {
+				$canvas->fillpoly([
+					$x - 2, $y - 2,
+					$x + DefBookmarkX - 4, $y - DefBookmarkX,
+					$x - 2, $y - DefBookmarkX,  
+				]);
+			} elsif ( $p > 0 && $a & 2 ) {
+				$canvas->fillpoly([
+					$x - 2, $y - 2,
+					$x + DefBookmarkX - 4, $y - DefBookmarkX,
+					$x + DefBookmarkX - 4, $y - 2,
+				]);
+			}
+		}
 
 		$canvas-> color( $c3d[0]);
 		$canvas-> line( 
@@ -1131,13 +1163,6 @@ sub on_paint
 		);
 
 		my $fh = 24;
-		my $a  = 0;
-		my ($pi, $mpi) = ( 
-			$self-> {notebook}-> pageIndex, 
-			$self-> {notebook}-> pageCount - 1
-		);
-		$a |= 1 if $pi > 0;
-		$a |= 2 if $pi < $mpi;
 		
 		$canvas-> line( 
 			DefBorderX + 4, $y - $fh * 1.6, 
@@ -1219,6 +1244,29 @@ sub on_mousedown
 	$self-> pageIndex( $self-> pageIndex + (( -$x + DefBookmarkX < $y) ? 1 : -1));
 }
 
+sub on_mousemove
+{
+	my ( $self, $mod, $x, $y) = @_;
+	my $prelight;
+	if (( $x, $y) = $self-> event_in_page_flipper( $x, $y)) {
+		if (-$x + DefBookmarkX < $y && $self->pageIndex < $self->pageCount - 1) {
+			$prelight = 1;
+		} elsif (-$x + DefBookmarkX >= $y && $self->pageIndex > 0 ){
+			$prelight = -1;
+		}
+	}
+	if (( $self->{prelight} // 0) != ($prelight // 0)) {
+		$self->{prelight} = $prelight;
+		$self->repaint;
+	}
+}
+
+sub on_mouseleave
+{
+	my $self = shift;
+	$self->repaint if delete $self->{prelight};
+}
+
 sub on_mousewheel
 {
 	my ( $self, $mod, $x, $y, $z) = @_;
