@@ -1,32 +1,3 @@
-#  Copyright (c) 1997-2002 The Protein Laboratory, University of Copenhagen
-#  All rights reserved.
-#
-#  Redistribution and use in source and binary forms, with or without
-#  modification, are permitted provided that the following conditions
-#  are met:
-#  1. Redistributions of source code must retain the above copyright
-#     notice, this list of conditions and the following disclaimer.
-#  2. Redistributions in binary form must reproduce the above copyright
-#     notice, this list of conditions and the following disclaimer in the
-#     documentation and/or other materials provided with the distribution.
-#
-#  THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
-#  ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
-#  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
-#  ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
-#  FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
-#  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
-#  OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
-#  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
-#  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
-#  OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
-#  SUCH DAMAGE.
-#
-#  Created by:
-#     Dmitry Karasik <dk@plab.ku.dk> 
-#
-#  $Id$
-
 use strict;
 use warnings;
 use Prima;
@@ -103,7 +74,7 @@ $OP_LINK = tb::opcode(1, 'link');
 
 # init a 'hand' pointer
 	$handIcon = Prima::Icon-> create( width=>19, height=>24, type => im::bpp4,
-	hScaling => 0, vScaling => 0, conversion => ict::None,
+	scaling => ist::None, conversion => ict::None,
 	palette => [ 0,0,0,0,0,128,0,128,0,0,128,128,128,0,0,128,0,128,128,128,0,128,128,128,192,192,192,0,0,255,0,255,0,0,255,255,255,0,0,0,0,0,255,255,0,255,255,255],
 	data => <<XDATA);
 3333\x00\x00\x00\x0330\x00\x003330\xff\xff\xff\x0330\x00\x003330\xff\xff\xff\x0330\x00\x00333\x0f\xff\xff\xff\xf030\x00\x00330\xff\xff\xff\xff\xf030\x00\x0033\x0f\xff\xff\xff\xff\xff\x030\x00\x0033\x0f\xff\xff\xff\xff\xff\x030\x00\x0030\xff\xff\xff\xff\xff\xff\x030\x00\x0030\xff\xff\xff\xff\xff\xff\xf00\x00\x003\x0f\xff\xff\xff\xff\xff\xff\xf00\x00\x003\x0f\xf0\xff\xff\xff\xff\xff\xf00\x00\x000\xff\x00\xff\xff\xff\xff\xff\xf00\x00\x000\xf00\xff\xff\xff\xff\xf0\xf00\x00\x00\x0f\x030\xff\xf0\xff\x0f\x0f\xf00\x00\x00\x0030\xff\x0f\xf0\xff\x0f\xf00\x00\x00330\xff\x0f\xf0\xff\x00\x030\x00\x00330\xff\x0f\xf0\xff\x0330\x00\x00330\xff\x0f\xf0\x00330\x00\x00330\xff\x00\x033330\x00\x00330\xff\x0333330\x00\x00330\xff\x0333330\x00\x00330\xff\x0333330\x00\x00330\xff\x0333330\x00\x00333\x00333330\x00\x00
@@ -633,6 +604,8 @@ sub open_read
 
 		createIndex   => 1,
 		encoding      => undef,
+		bom           => undef,
+		utf8          => undef,
 
 		@opt,
 	};
@@ -649,12 +622,12 @@ sub add_formatted
 		$self-> add_new_line;
 	} elsif ( $format eq 'podview') {
 		while ( $text =~ m/<\s*([^<>]*)\s*>/gcs) {
-			my $cmd = lc $1;
-			if ( $cmd eq 'cut') {
+			my $cmd = $1;
+			if ( lc($cmd) eq 'cut') {
 				$self-> {readState}-> {pod_cutting} = 0;
-			} elsif ( $cmd eq '/cut') {
+			} elsif ( lc($cmd) eq '/cut') {
 				$self-> {readState}-> {pod_cutting} = 1;
-			} elsif ( $cmd =~ /^img\s*(.*)$/) {
+			} elsif ( $cmd =~ /^img\s*(.*)$/i) {
 				$cmd = $1;
 				my ( $w, $h, $src, $frame, $cut);
 				$frame = 0;
@@ -837,6 +810,13 @@ sub read
 	my $r = $self-> {readState};
 	return unless $r;
 
+	unless ( defined $r->{bom} ) {
+		if ( $pod =~ s/^(\x{ef}\x{bb}\x{bf})// ) { # don't care about other BOMs so far
+			$r-> {bom} = $1;
+			$r-> {encoding} = Encode::find_encoding('utf-8');
+		}
+	}
+
 	my $odd = 0;
 	for ( split ( "(\n)", $pod)) {
 		next unless $odd = !$odd;
