@@ -1,35 +1,7 @@
-#
-#  Copyright (c) 1997-2002 The Protein Laboratory, University of Copenhagen
-#  All rights reserved.
-#
-#  Redistribution and use in source and binary forms, with or without
-#  modification, are permitted provided that the following conditions
-#  are met:
-#  1. Redistributions of source code must retain the above copyright
-#     notice, this list of conditions and the following disclaimer.
-#  2. Redistributions in binary form must reproduce the above copyright
-#     notice, this list of conditions and the following disclaimer in the
-#     documentation and/or other materials provided with the distribution.
-#
-#  THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
-#  ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
-#  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
-#  ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
-#  FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
-#  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
-#  OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
-#  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
-#  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
-#  OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
-#  SUCH DAMAGE.
-#
 #  Created by:
 #     Anton Berezin  <tobez@tobez.org>
 #     Dmitry Karasik <dk@plab.ku.dk> 
 #
-#  $Id: StdDlg.pm,v 1.27 2005/10/13 17:22:51 dk Exp $
-
-
 
 # Contains stubs for load-on-demand of the following modules:
 #   Prima::OpenDialog       => Prima/FileDialog.pm
@@ -43,7 +15,10 @@
 #   Prima::ImageOpenDialog  => Prima/ImageDialog.pm
 #   Prima::ImageSaveDialog  => Prima/ImageDialog.pm
 
-no strict;
+package Prima::StdDlg;
+use strict;
+use warnings;
+
 
 package Prima::ColorDialog;
 
@@ -136,7 +111,8 @@ sub AUTOLOAD
 	shift-> $method(@_);
 }
 
-package mb;
+package 
+    mb;
 
 use constant ChangeAll => 0xCA11;
 
@@ -212,7 +188,7 @@ no need to C<use> the corresponding module explicitly.
 =head1 SYNOPSIS
 
 	use Prima::StdDlg;
-	
+
 	Prima::FileDialog-> create-> execute;
 	Prima::FontDialog-> create-> execute;
 
