@@ -1,36 +1,8 @@
-#
-#  Copyright (c) 1997-2002 The Protein Laboratory, University of Copenhagen
-#  All rights reserved.
-#
-#  Redistribution and use in source and binary forms, with or without
-#  modification, are permitted provided that the following conditions
-#  are met:
-#  1. Redistributions of source code must retain the above copyright
-#     notice, this list of conditions and the following disclaimer.
-#  2. Redistributions in binary form must reproduce the above copyright
-#     notice, this list of conditions and the following disclaimer in the
-#     documentation and/or other materials provided with the distribution.
-#
-#  THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
-#  ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
-#  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
-#  ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
-#  FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
-#  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
-#  OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
-#  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
-#  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
-#  OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
-#  SUCH DAMAGE.
-#
-#  $Id$
-#
-
 =pod
 
 =head1 NAME
 
-A basic text editor
+examples/editor.pl - A basic text editor
 
 =head1 FEATURES
 
@@ -44,6 +16,7 @@ use warnings;
 
 use Prima;
 use Prima::Edit;
+use Prima::Bidi qw(:enable);
 use Prima::Application;
 use Prima::MsgBox;
 use Prima::StdDlg;
@@ -403,9 +376,9 @@ sub do_find
 		if ( $$p{scope} != fds::Cursor) {
 			if ( $e-> has_selection) {
 			my @sel = $e-> selection;
-			@scope = ($$p{scope} == fds::Top) ? ($sel[0],$sel[1]) : ($sel[2], $sel[3]);
+				@scope = ($$p{scope} == fds::Top) ? ($sel[0],$sel[1]) : ($sel[2], $sel[3]);
 			} else {
-			@scope = ($$p{scope} == fds::Top) ? (0,0) : (-1,-1);
+				@scope = ($$p{scope} == fds::Top) ? (0,0) : (-1,-1);
 			}
 		} else {
 			@scope = $e-> cursor;
@@ -415,8 +388,8 @@ sub do_find
 			Prima::MsgBox::message("No matches found");
 			return;
 		}
-		$e-> cursor(($$p{options} & fdo::BackwardSearch) ? $n[0] : $n[0] + $n[2], $n[1]);
-		$e-> selection( $n[0], $n[1], $n[0] + $n[2], $n[1]);
+		$e-> cursor(($$p{options} & fdo::BackwardSearch) ? $n[0] : $n[2], $n[1]);
+		$e-> selection( $n[0], $n[1], $n[2], $n[1]);
 		unless ( $$p{asFind}) {
 			if ( $$p{options} & fdo::ReplacePrompt) {
 				my $r = Prima::MsgBox::message_box( $self-> text,
@@ -471,9 +444,13 @@ my @fn = @ARGV;
 
 for ( @fn) {
 my $w = EditorWindow-> create(
-	origin => [ 10, 100],
-	size   => [ $::application-> width - 820, $::application-> height - 150],
+        origin => [ 10, 100],
+        size   => [ $::application-> width - 820, $::application-> height - 150],
 	fileName => $_,
+	font => {
+		size => 16,
+		name => 'Courier New',
+	},
 );
 }
 
