@@ -1,31 +1,3 @@
-/*
- * Copyright (c) 1997-2002 The Protein Laboratory, University of Copenhagen
- * All rights reserved.
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer.
- * 2. Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in the
- *    documentation and/or other materials provided with the distribution.
- *
- * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
- * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
- * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
- * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
- * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
- * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
- * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
- * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
- * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
- * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
- * SUCH DAMAGE.
- *
- * $Id: apc.c,v 1.118 2005/10/11 20:58:57 dk Exp $
- */
-/* Created by Dmitry Karasik <dk@plab.ku.dk> */
 #include "win32\win32guts.h"
 #include <commdlg.h>
 #ifndef _APRICOT_H_
@@ -89,20 +61,27 @@ apc_application_create( Handle self)
 {
    HWND h;
    RECT r;
+   MSG msg;
+   const WCHAR wnull = 0;
+
    objCheck false;
-   if ( !( h = CreateWindowEx( 0, "GenericApp", "", 0, 0, 0, 0, 0,
+
+   // make sure that no leftover messages, esp.WM_QUIT, are floating around
+   while ( PeekMessage( &msg, NULL, 0, 0, PM_REMOVE)); 
+
+   if ( !( h = CreateWindowExW( 0, L"GenericApp", &wnull, 0, 0, 0, 0, 0,
           nil, nil, guts. instance, nil))) apiErrRet;
    sys handle = h;
    sys parent = sys owner = HWND_DESKTOP;
-   SetWindowLong( sys handle, GWL_USERDATA, self);
+   SetWindowLongPtr( sys handle, GWLP_USERDATA, self);
    PostMessage( sys handle, WM_PRIMA_CREATE, 0, 0);
    sys className = WC_APPLICATION;
    // if ( !SetTimer( h, TID_USERMAX, 100, nil)) apiErr;
    GetClientRect( h, &r);
-   if ( !( var handle = ( Handle) CreateWindowEx( 0,  "Generic", "", WS_VISIBLE | WS_CHILD | WS_CLIPCHILDREN,
+   if ( !( var handle = ( Handle) CreateWindowExW( 0,  L"Generic", &wnull, WS_VISIBLE | WS_CHILD | WS_CLIPCHILDREN,
         0, 0, r. right - r. left, r. bottom - r. top, h, nil,
         guts. instance, nil))) apiErrRet;
-   SetWindowLong(( HWND) var handle, GWL_USERDATA, self);
+   SetWindowLongPtr(( HWND) var handle, GWLP_USERDATA, self);
    apt_set( aptEnabled);
    sys lastSize = apc_application_get_size( self);
    return true;
@@ -119,7 +98,7 @@ Bool
 apc_application_destroy( Handle self)
 {
    objCheck false;
-   SetWindowLong( sys handle, GWL_USERDATA, 0);
+   SetWindowLongPtr( sys handle, GWLP_USERDATA, 0);
    if ( IsWindow( sys handle))  {
       if ( guts. mouseTimer) {
           guts. mouseTimer = 0;
@@ -215,6 +194,8 @@ apc_application_get_bitmap( Handle self, Handle image, int x, int y, int xLen, i
    SelectPalette( dc, hp3, 1);
    dc_free();
 
+   apc_image_update_change( image);
+
    return true;
 }
 
@@ -223,18 +204,18 @@ Handle
 hwnd_to_view( HWND win)
 {
    Handle h;
-   LONG ll;
+   LONG_PTR ll;
    if (( !win) || ( !IsWindow( win)))
       return nilHandle;
    if ( GetWindowThreadProcessId( win, nil) != guts. mainThreadId)
       return nilHandle;
-   h = GetWindowLong( win, GWL_USERDATA);
+   h = GetWindowLongPtr( win, GWLP_USERDATA);
    if ( !h) return nilHandle;
-   ll = GetWindowLong( win, GWL_WNDPROC);
+   ll = GetWindowLongPtr( win, GWLP_WNDPROC);
    if (
-       ( ll == ( LONG) generic_view_handler) ||
-       ( ll == ( LONG) generic_app_handler) ||
-       ( ll == ( LONG) generic_frame_handler)
+       ( ll == ( LONG_PTR) generic_view_handler) ||
+       ( ll == ( LONG_PTR) generic_app_handler) ||
+       ( ll == ( LONG_PTR) generic_frame_handler)
       ) return h;
 
    if ( SendMessage( win, WM_HASMATE, 0, ( LPARAM) &h) == HASMATE_MAGIC)
@@ -256,19 +237,7 @@ apc_application_get_os_info( char *system, int slen,
    version = GetVersion();
    GetVersionEx( &os);
    if ( system) {
-      if ( IS_NT) {
-         strncpy( system, "Windows NT", slen);
-      } else if ( IS_WIN95) {
-         if ((os.dwPlatformId == VER_PLATFORM_WIN32_WINDOWS) &&
-             ((os.dwMajorVersion > 4) ||
-              ((os.dwMajorVersion == 4) && (os.dwMinorVersion > 0)))) {
-            strncpy( system, "Windows 98", slen);
-         } else {
-            strncpy( system, "Windows 95", slen);
-         }
-      } else {
-         strncpy( system, "Windows", slen);
-      }
+      strncpy( system, "Windows NT", slen);
       system[ slen-1] = 0;
    }
    if ( vendor) {
@@ -303,6 +272,46 @@ apc_application_get_handle( Handle self, ApiHandle apiHandle)
    return hwnd_to_view(( HWND) apiHandle);
 }
 
+Rect
+apc_application_get_indents( Handle self)
+{
+	Point size;
+	UINT rc;
+	Rect ret = {0,0,0,0};
+	APPBARDATA d;
+	
+	size = apc_application_get_size( self);
+
+	memset( &d, 0, sizeof(d));
+	d. cbSize = sizeof(d);
+	rc = SHAppBarMessage( ABM_GETSTATE, &d);
+	if (( rc & ABS_AUTOHIDE) == 0) {
+		memset( &d, 0, sizeof(d));
+		d. cbSize = sizeof(d);
+		rc = SHAppBarMessage( ABM_GETTASKBARPOS, &d);
+		switch ( d. uEdge) {
+		case ABE_TOP:
+			ret. top = d. rc. bottom;
+			if ( ret. top < 0) ret. top = 0;
+			break;
+		case ABE_BOTTOM:
+			ret. bottom = size. y - d. rc. top;
+			if ( ret. bottom < 0) ret. bottom = 0;
+			break;
+		case ABE_RIGHT:
+			ret. right = size. x - d. rc. left;
+			if ( ret. right < 0) ret. right = 0;
+			break;
+		case ABE_LEFT:
+			ret. left = d. rc. right;
+			if ( ret. left < 0) ret. left = 0;
+			break;
+		}
+	}
+	
+	return ret;
+}
+
 Point
 apc_application_get_size( Handle self)
 {
@@ -315,6 +324,29 @@ apc_application_get_size( Handle self)
    return ret;
 }
 
+Rect2 *
+apc_application_get_monitor_rects( Handle self, int * nrects)
+{
+    *nrects = 0;
+    return nil;
+}
+
+static void
+apcDispatchMessage( const MSG * lpmsg)
+{
+    DispatchMessage(lpmsg);
+    exception_check_raise();
+}
+
+int
+apcUpdateWindow( HWND win )
+{
+    int ret;
+    ret = UpdateWindow( win );
+    exception_check_raise();
+    return ret;
+}
+
 static Bool
 files_rehash( Handle self, void * dummy)
 {
@@ -355,6 +387,7 @@ process_msg( MSG * msg)
          BYTE * mod = mod_select( kp-> mod);
          SendMessage( kp-> wnd, kp-> msg, kp-> mp1, kp-> mp2);
          mod_free( mod);
+         exception_check_raise();
       }
       break;
    case WM_LBUTTONDOWN:  musClk. emsg = WM_LBUTTONUP; goto MUS1;
@@ -439,7 +472,7 @@ process_msg( MSG * msg)
    }
    if ( !postpone_msg_translation) 
       TranslateMessage( msg);
-   DispatchMessage( msg);
+   apcDispatchMessage( msg);
    if ( postpone_msg_translation) { 
       if ( guts. dont_xlate_message)
          guts. dont_xlate_message = false;
@@ -456,7 +489,7 @@ apc_application_go( Handle self)
 {
    MSG msg;
    objCheck false;
-
+  
    while ( GetMessage( &msg, NULL, 0, 0) && process_msg( &msg));
 
    if ( application) Object_destroy( application);
@@ -490,13 +523,21 @@ apc_application_unlock( Handle self)
    return HWND_lock( false);
 }
 
+Bool
+apc_application_sync( void)
+{
+   return true;
+}
+
 Bool
 apc_application_yield()
 {
    MSG msg;
    while ( PeekMessage( &msg, NULL, 0, 0, PM_REMOVE))
-      if ( !process_msg( &msg))
+      if ( !process_msg( &msg)) {
          PostThreadMessage( guts. mainThreadId, appDead ? WM_QUIT : WM_TERMINATE, 0, 0);
+         break;
+      }
    return true;
 }
 
@@ -522,7 +563,7 @@ apc_application_get_widget_from_point( Handle self, Point point)
    if ( !p) return nilHandle;
    if ( !( tid = GetWindowThreadProcessId( p, &pid))) apiErr;
    if ( tid != guts. mainThreadId) return nilHandle;
-   return ( Handle) GetWindowLong( p, GWL_USERDATA);
+   return ( Handle) GetWindowLongPtr( p, GWLP_USERDATA);
 }
 
 // Component
@@ -603,8 +644,13 @@ get_view_ex( Handle self, PViewProfile p)
   if ( !p) return;
   p-> capture   = apc_widget_is_captured( self);
   for ( i = 0; i <= ciMaxId; i++) p-> colors[ i] = apc_widget_get_color( self, i);
-  p-> pos       = apc_widget_get_pos( self);
-  p-> size      = apc_widget_get_size( self);
+  if ( sys className == WC_FRAME) {
+     p-> pos       = apc_window_get_client_pos( self);
+     p-> size      = apc_window_get_client_size( self);
+  } else {
+     p-> pos       = apc_widget_get_pos( self);
+     p-> size      = apc_widget_get_size( self);
+  }
   p-> virtSize  = var virtualSize;
   p-> enabled   = apc_widget_is_enabled( self);
   p-> focused   = apc_widget_is_focused( self);
@@ -620,7 +666,11 @@ set_view_ex( Handle self, PViewProfile p)
   apc_widget_set_visible( self, false);
   for ( i = 0; i <= ciMaxId; i++) apc_widget_set_color( self, p-> colors[i], i);
   apc_widget_set_font( self, &var font);
-  apc_widget_set_rect( self, p-> pos. x, p-> pos. y, p-> size.x, p-> size.y);
+  if ( sys className == WC_FRAME) {
+     apc_window_set_client_rect( self, p-> pos. x, p-> pos. y, p-> size.x, p-> size.y);
+  } else {
+     apc_widget_set_rect( self, p-> pos. x, p-> pos. y, p-> size.x, p-> size.y);
+  }
   var virtualSize = p-> virtSize;
   apc_widget_set_enabled( self, p-> enabled);
   if ( p-> focused) apc_widget_set_focused( self);
@@ -661,6 +711,7 @@ create_group( Handle self, Handle owner, Bool syncPaint, Bool clipOwner,
    int  count = 0;
    Bool reset = false;
    Handle * list = nil;
+   const WCHAR wnull = 0;
 
    if ( HANDLE)
    {
@@ -696,7 +747,7 @@ create_group( Handle self, Handle owner, Bool syncPaint, Bool clipOwner,
               parentView = DHANDLE( application);
           if ( !usePos)  rcp[0] = rcp[1] = CW_USEDEFAULT;
           if ( !useSize) rcp[2] = rcp[3] = CW_USEDEFAULT;
-          if ( !( frame = CreateWindowEx( exstyle, "GenericFrame", "",
+          if ( !( frame = CreateWindowExW( exstyle, L"GenericFrame", &wnull,
                 style | WS_CLIPCHILDREN,
                 rcp[0], rcp[1], rcp[2], rcp[3],
                 parentView, nil, guts. instance, nil)))
@@ -710,7 +761,7 @@ create_group( Handle self, Handle owner, Bool syncPaint, Bool clipOwner,
                    0,0,0,0,SWP_NOMOVE | SWP_NOSIZE  | SWP_NOACTIVATE))
                 apiErr;
           GetClientRect( frame, &r);
-          if ( !( ret = CreateWindowEx( 0,  "Generic", "",
+          if ( !( ret = CreateWindowExW( 0,  L"Generic", &wnull,
                 WS_VISIBLE | WS_CHILD | WS_CLIPCHILDREN | WS_CLIPSIBLINGS,
                 0, 0, r. right - r. left, r. bottom - r. top, frame, nil,
                 guts. instance, nil)))
@@ -718,7 +769,7 @@ create_group( Handle self, Handle owner, Bool syncPaint, Bool clipOwner,
           sys lastSize. x = r. right  - r. left;
           sys yOverride = sys lastSize. y = r. bottom - r. top;
           sys handle = frame;
-          SetWindowLong( frame, GWL_USERDATA, ( LONG) self);
+          SetWindowLongPtr( frame, GWLP_USERDATA, ( LONG) self);
        }
        break;
     case WC_CUSTOM:
@@ -730,7 +781,7 @@ create_group( Handle self, Handle owner, Bool syncPaint, Bool clipOwner,
        if ( parentHandle) parentView = parentHandle;
        sys parentHandle = parentHandle;
 
-       if ( !( ret = CreateWindowEx( exstyle,  "Generic", "",
+       if ( !( ret = CreateWindowExW( exstyle,  L"Generic", &wnull,
              style | WS_CLIPCHILDREN | WS_CLIPSIBLINGS, 0, 0, 0, 0,
              parentView, nil, guts. instance, nil)))
           apiErrRet;
@@ -749,13 +800,17 @@ create_group( Handle self, Handle owner, Bool syncPaint, Bool clipOwner,
    sys owner   = ownerView;
    if ( !reset)
       sys pointer = LoadCursor( guts. instance, IDC_ARROW);
-   SetWindowLong( ret, GWL_USERDATA, ( LONG) self);
+   SetWindowLongPtr( ret, GWLP_USERDATA, ( LONG) self);
 
    if ( reset)
    {
       int i;
       Handle oldOwner = var owner; var owner = owner;
-      apc_widget_set_rect( self, vprf-> pos. x, vprf-> pos. y, vprf-> size. x, vprf-> size. y);
+      if ( sys className == WC_FRAME) {
+         apc_window_set_client_rect( self, vprf-> pos. x, vprf-> pos. y, vprf-> size. x, vprf-> size. y);
+      } else {
+         apc_widget_set_rect( self, vprf-> pos. x, vprf-> pos. y, vprf-> size. x, vprf-> size. y);
+      }
       var owner = oldOwner;
       for ( i = 0; i < count; i++) ((( PComponent) list[ i])-> self)-> recreate( list[ i]);
       if ( sys className == WC_FRAME)
@@ -808,6 +863,7 @@ apc_window_create( Handle self, Handle owner, Bool syncPaint, int borderIcons,
   ViewProfile vprf;
   int oStage = var stage;
   WindowData ws;
+  HICON icon = (HICON) nilHandle;
   WINDOWPLACEMENT wp = {sizeof(WINDOWPLACEMENT)};
   DWORD style = WS_CLIPCHILDREN | WS_OVERLAPPED
      | (( borderIcons &  biSystemMenu) ? WS_SYSMENU     : 0)
@@ -848,6 +904,7 @@ apc_window_create( Handle self, Handle owner, Bool syncPaint, int borderIcons,
      ws = sys s. window;
      if ( !GetWindowPlacement( HANDLE, &wp)) apiErr;
      usePos = useSize = 1; // prevent using shell-position flags for recreate
+     icon = ( HICON) SendMessage( HANDLE, WM_GETICON, ICON_BIG, 0);
      reset = true;
   }
   HWND_lock( true);
@@ -872,13 +929,18 @@ apc_window_create( Handle self, Handle owner, Bool syncPaint, int borderIcons,
   if ( usePos) apt_set( aptWinPosDetermined);
   if ( reset)
   {
-     Handle oldOwner = var owner; var owner = owner;
+     Handle oldOwner = var owner; 
+     
+     var owner = owner;
      apc_window_set_window_state( self, windowState);
      var owner = oldOwner;
      set_view_ex( self, &vprf);
      sys s. window = ws;
+     if ( windowState != wsMaximized)
+        GetWindowRect( HANDLE, &wp. rcNormalPosition);
      if ( !SetWindowPlacement( HANDLE, &wp)) apiErr;
      var stage = oStage;
+     if ( icon) SendMessage( HANDLE, WM_SETICON, ICON_BIG, ( LPARAM) icon);
   }
   else {
 //   WINDOWPLACEMENT wp = {sizeof( WINDOWPLACEMENT)};
@@ -956,6 +1018,13 @@ apc_window_get_border_style( Handle self)
    return sys s. window. borderStyle;
 }
 
+ApiHandle
+apc_window_get_client_handle( Handle self)
+{
+   objCheck 0;
+   return ( ApiHandle) var handle;
+}
+
 Point
 apc_window_get_client_pos( Handle self)
 {
@@ -1033,45 +1102,56 @@ map_tildas( WCHAR * buf, int len)
              buf[ j] = '&';
           continue;
       } else if ( buf[ j] == '&') {
-         memmove( buf + j + 1, buf + j, (len - j - 1) * sizeof( WCHAR));
+         memmove( buf + j + 1, buf + j, (len - j + 1) * sizeof( WCHAR));
          j++;
+         len++;
          continue;
       }
    }
-   buf[len - 1] = 0;
 }
-
+      
 static WCHAR *
 map_text_accel( PMenuItemReg i)
 {
    char * c;
    int l1, l2 = 0, amps = 0;
-   WCHAR * buf;
+   WCHAR *buf, *text, *accel;
 
-   l1 = 1 + (( i-> flags. utf8_text && HAS_WCHAR) ? prima_utf8_length( i-> text) : strlen( i-> text));
-   c = i-> text;  while (*c++) if ( *c == '&') amps++;
-   if ( i-> accel) {
-      l2 = 1 + (( i-> flags. utf8_accel && HAS_WCHAR) ? prima_utf8_length( i-> accel) : strlen( i-> accel));
-      c = i-> accel; while (*c++) if ( *c == '&') amps++;
+   c = i-> text;
+   while (*c++) if ( *c == '&') amps++;
+   if ( i-> flags. utf8_text ) {
+      text = alloc_utf8_to_wchar( i-> text, prima_utf8_length( i-> text), &l1);
+   } else {
+      l1 = strlen( i-> text);
+      text = alloc_ascii_to_wchar( i-> text, l1);
+   }      
+  
+   if ( i-> accel ) {
+      c = i-> accel; 
+      while (*c++) if ( *c == '&') amps++;
+      if ( i-> flags. utf8_accel ) {
+         accel = alloc_utf8_to_wchar( i-> accel, prima_utf8_length( i-> accel), &l2);
+      } else {
+         l2 = strlen( i-> accel);
+         accel = alloc_ascii_to_wchar( i-> accel, l2);
+      }
+   }
+
+   buf = malloc(sizeof(WCHAR) * (l1 + l2 + amps + 2));
+
+   memcpy( buf, text, l1 * sizeof(WCHAR));
+   free(text);
+
+   if ( i->accel ) {
+      buf[l1] = '\t';
+      memcpy( buf + l1 + 1, accel, l2 * sizeof(WCHAR));
+      free(accel);
+      l2++;
    }
-   buf = malloc( sizeof( WCHAR) * ( l1 + l2 + amps));
-   if ( !buf) return nil;
-   memset( buf, 0, sizeof( WCHAR) * ( l1 + l2 + amps));
    
-   if ( i-> flags. utf8_text && HAS_WCHAR) 
-      utf8_to_wchar( i-> text, buf, l1 - 1);
-   else
-      MultiByteToWideChar( CP_ACP, MB_PRECOMPOSED, i-> text, l1 - 1, buf, l1 * 2 - 2);
-   if ( i-> accel) {
-      buf[l1 - 1] = '\t';
-      if ( i-> flags. utf8_accel && HAS_WCHAR) 
-         utf8_to_wchar( i-> accel, buf + l1, l2);
-      else
-         MultiByteToWideChar( CP_ACP, MB_PRECOMPOSED, i-> accel, l2 - 1, buf + l1, l2 * 2 - 2);
-   }
-   map_tildas( buf, l1 + l2 + amps);
-   if ( !HAS_WCHAR)
-      wchar2char(( char*) buf, buf, l1 + l2 + amps); 
+   buf[l1 + l2] = 0;
+   map_tildas( buf, l1 + l2);
+   
    return buf;
 }
 
@@ -1123,10 +1203,7 @@ add_item( Bool menuType, Handle menu, PMenuItemReg i)
              menuItem. dwTypeData = map_text_accel( i);
           } else if ( i-> bitmap && PObject( i-> bitmap)-> stage < csDead)
              menuItem. dwTypeData = ( LPWSTR) image_make_bitmap_handle( i-> bitmap, nil);
-          if ( HAS_WCHAR)
-             InsertMenuItemW( m, -1, true, &menuItem);
-          else
-             InsertMenuItemA( m, -1, true, ( LPMENUITEMINFOA) &menuItem);
+          InsertMenuItemW( m, -1, true, &menuItem);
           if ( i-> text && menuItem. dwTypeData) free( menuItem. dwTypeData);
        }
        menuItem. dwItemData = menu;
@@ -1166,12 +1243,12 @@ Bool
 apc_window_set_caption( Handle self, const char * caption, Bool utf8)
 {
    objCheck false;
-   if ( HAS_WCHAR && utf8) {
-      WCHAR * c = alloc_utf8_to_wchar( caption, -1);
+   if ( utf8) {
+      WCHAR * c = alloc_utf8_to_wchar( caption, -1, NULL);
       if ( !( rc = SetWindowTextW( HANDLE, c))) apiErr;
       free( c);
    } else {
-      if ( !( rc = SetWindowText( HANDLE, caption))) apiErr;
+      if ( !( rc = SetWindowTextA( HANDLE, caption))) apiErr;
    }
    return rc == 0;
 }
@@ -1348,7 +1425,7 @@ apc_window_set_icon( Handle self, Handle icon)
 {
    HICON i;
    objCheck false;
-   i = icon ? image_make_icon_handle( icon, guts. iconSizeLarge, nil, false) : nil;
+   i = icon ? image_make_icon_handle( icon, guts. iconSizeLarge, nil) : nil;
    i = ( HICON) SendMessage( HANDLE, WM_SETICON, ICON_BIG, ( LPARAM) i);
    if ( i) DestroyIcon( i);
    return true;
@@ -1570,12 +1647,12 @@ apc_widget_begin_paint( Handle self, Bool insideOnPaint)
             MSG  msg;
             list_add( &guts. transp, self);
             WinHideWindow(( HWND) var handle);
-            if ( parent) UpdateWindow( parent);
+            if ( parent) apcUpdateWindow( parent);
             while ( PeekMessage( &msg, NULL, WM_PAINT, WM_PAINT, PM_REMOVE))
-               DispatchMessage( &msg);
+               apcDispatchMessage( &msg);
             if ( !parent) Sleep( 1);
             WinShowWindow(( HWND) var handle);
-            UpdateWindow(( HWND) var handle);
+            apcUpdateWindow(( HWND) var handle);
             list_delete( &guts. transp, self);
          } else
             useRPDraw = true;
@@ -1645,10 +1722,10 @@ apc_widget_begin_paint( Handle self, Bool insideOnPaint)
          RealizePalette( sys ps);
       }
    }
-   hwnd_enter_paint( self);
 
    if ( useRPDraw) {
       HDC dc;
+      HGDIOBJ o_pen, o_brush, o_font, o_pal, o_extpen;
       Handle owner = var owner;
       Point tr = dsys(owner)transform2;
       Point ed = apc_widget_get_pos( self);
@@ -1661,12 +1738,31 @@ apc_widget_begin_paint( Handle self, Bool insideOnPaint)
       dsys(owner) transform2. x += ed. x;
       dsys(owner) transform2. y += so. y - sz. y - ed. y;
       apc_gp_set_transform( owner, 0, 0);
+      apc_gp_set_text_out_baseline( owner, dsys(owner) options. aptTextOutBaseline);
+
+      SelectObject( sys ps, o_pen    = GetCurrentObject( dc, OBJ_PEN));
+      SelectObject( sys ps, o_brush  = GetCurrentObject( dc, OBJ_BRUSH));
+      SelectObject( sys ps, o_font   = GetCurrentObject( dc, OBJ_FONT));
+      SelectObject( sys ps, o_pal    = GetCurrentObject( dc, OBJ_PAL));
+      SelectObject( sys ps, o_extpen = GetCurrentObject( dc, OBJ_EXTPEN));
 
       CWidget( owner)-> notify( owner, "sH", "Paint", owner);
+
+      SelectObject( sys ps, o_pen);
+      SelectObject( sys ps, o_brush);
+      SelectObject( sys ps, o_font);
+      SelectObject( sys ps, o_pal);
+      SelectObject( sys ps, o_extpen);
+
       dsys(owner)transform2 = tr;
       apc_gp_set_transform( owner, 0, 0);
       dsys( owner) ps = dc;
       CWidget( owner)-> end_paint( owner);
+   }
+
+   hwnd_enter_paint( self);
+
+   if ( useRPDraw) {
       apc_gp_set_transform( self, sys transform. x, sys transform. y);
    }
 
@@ -1695,7 +1791,7 @@ Bool
 apc_widget_destroy( Handle self)
 {
    objCheck false;
-   SetWindowLong( HANDLE, GWL_USERDATA, 0);
+   SetWindowLongPtr( HANDLE, GWLP_USERDATA, 0);
    if ( sys pointer2) {
       if ( sys pointer2 == sys pointer) SetCursor( NULL); // un-use resource first
       if ( !DestroyCursor( sys pointer2)) apiErr;
@@ -2103,19 +2199,20 @@ apc_widget_invalidate_rect( Handle self, Rect * rect)
    PRECT pRect = rect ? map_Rect( self, rect) : nil;
    objCheck false;
    if ( !InvalidateRect (( HWND) var handle, pRect, false)) apiErr;
-   if ( is_apt( aptSyncPaint) && !UpdateWindow(( HWND) var handle)) apiErr;
+   if ( is_apt( aptSyncPaint) && !apcUpdateWindow(( HWND) var handle)) apiErr;
    objCheck false;
    process_transparents( self);
    return true;
 }
 
-Bool
+int
 apc_widget_scroll( Handle self, int horiz, int vert, Rect * r, Rect *cr, Bool scrollChildren)
 {
    PRECT pRect = r ? map_Rect( self, r) : nil;
    PRECT pClipRect = cr ? map_Rect( self, cr) : nil;
    Point sz = apc_widget_get_size( self);
-   objCheck false;
+   int ret;
+   objCheck scrError;
 
    HideCaret(( HWND) var handle);
 
@@ -2140,18 +2237,21 @@ apc_widget_scroll( Handle self, int horiz, int vert, Rect * r, Rect *cr, Bool sc
          InvalidateRect(( HWND) var handle, &rc, false);
       } else 
          InvalidateRect(( HWND) var handle, pRect ? pRect : pClipRect, false);
+      ret = scrExpose;
    } else {
-      if ( !ScrollWindowEx(( HWND) var handle,
+      int rr = ScrollWindowEx(( HWND) var handle,
          horiz, -vert, pRect, pClipRect, NULL, NULL,
          SW_INVALIDATE | ( scrollChildren ? SW_SCROLLCHILDREN : 0)
-      )) {
-         ShowCaret(( HWND) var handle);
-         apiErr;
-      }
+      ); 
+      if (!rr) apiErr;
+      ret = ( rr == 1 ) ? scrNoExpose : scrExpose;
    }
-   objCheck false;
-   if ( is_apt( aptSyncPaint) && !UpdateWindow(( HWND) var handle)) apiErr;
-   return ShowCaret(( HWND) var handle);
+
+   objCheck scrError;
+   if ( is_apt( aptSyncPaint) && !apcUpdateWindow(( HWND) var handle)) apiErr;
+   ShowCaret(( HWND) var handle);
+
+   return ret;
 }
 
 Bool
@@ -2469,7 +2569,7 @@ Bool
 apc_widget_update( Handle self)
 {
    objCheck false;
-   if ( !UpdateWindow(( HWND) var handle)) apiErrRet;
+   if ( !apcUpdateWindow(( HWND) var handle)) apiErrRet;
    return true;
 }
 
@@ -2556,7 +2656,7 @@ apc_menu_set_font( Handle self, PFont font)
 Bool
 apc_menu_item_delete( Handle self, PMenuItemReg m)
 {
-   PWindow owner = nilHandle;
+   PWindow owner = nil;
    Point size;
    Bool resize;
    objCheck false;
@@ -2595,15 +2695,9 @@ apc_menu_item_set_accel( Handle self, PMenuItemReg m)
 
    apcErrClear;
    buf = map_text_accel( m);
-   if ( HAS_WCHAR) {
-      if ( !ModifyMenuW(( HMENU) var handle, m-> id + MENU_ID_AUTOSTART, flags,
-                       m-> id + MENU_ID_AUTOSTART, buf)) 
-         apiErr;
-   } else {
-      if ( !ModifyMenuA(( HMENU) var handle, m-> id + MENU_ID_AUTOSTART, flags,
-                       m-> id + MENU_ID_AUTOSTART, ( char *) buf)) 
-         apiErr;
-   }
+   if ( !ModifyMenuW(( HMENU) var handle, m-> id + MENU_ID_AUTOSTART, flags,
+                    m-> id + MENU_ID_AUTOSTART, buf)) 
+      apiErr;
    free( buf);
    return rc == errOk;
 }
@@ -2653,13 +2747,8 @@ apc_menu_item_set_text( Handle self, PMenuItemReg m)
 
    apcErrClear;
    buf = map_text_accel( m);
-   if ( HAS_WCHAR) {
-      if ( !ModifyMenuW(( HMENU) var handle, m-> id + MENU_ID_AUTOSTART, flags,
-                    m-> id + MENU_ID_AUTOSTART, (WCHAR*) buf)) apiErr;
-   } else {
-      if ( !ModifyMenuA(( HMENU) var handle, m-> id + MENU_ID_AUTOSTART, flags,
-                    m-> id + MENU_ID_AUTOSTART, (char*) buf)) apiErr;
-   }
+   if ( !ModifyMenuW(( HMENU) var handle, m-> id + MENU_ID_AUTOSTART, flags,
+                 m-> id + MENU_ID_AUTOSTART, (WCHAR*) buf)) apiErr;
    free( buf);
    return rc == errOk;
 }
@@ -2677,15 +2766,9 @@ apc_menu_item_set_image( Handle self, PMenuItemReg m)
 
    flags |= MF_BITMAP;
 
-   if ( HAS_WCHAR) {
-      if ( !ModifyMenuW(( HMENU) var handle, m-> id + MENU_ID_AUTOSTART, flags,
-                       m-> id + MENU_ID_AUTOSTART, 
-                       ( LPCWSTR) image_make_bitmap_handle( m-> bitmap, nil))) apiErrRet;
-   } else {
-      if ( !ModifyMenuA(( HMENU) var handle, m-> id + MENU_ID_AUTOSTART, flags,
-                       m-> id + MENU_ID_AUTOSTART, 
-                       ( LPCSTR) image_make_bitmap_handle( m-> bitmap, nil))) apiErrRet;
-   }
+   if ( !ModifyMenuW(( HMENU) var handle, m-> id + MENU_ID_AUTOSTART, flags,
+                    m-> id + MENU_ID_AUTOSTART, 
+                    ( LPCWSTR) image_make_bitmap_handle( m-> bitmap, nil))) apiErrRet;
    return true;
 }
 
@@ -2789,7 +2872,7 @@ apc_popup( Handle self, int x, int y, Rect * anchor)
 }
 
 
-int ctx_kb2VK[] = {
+Handle ctx_kb2VK[] = {
    kbNoKey       ,   0                 ,
    kbAltL        ,   VK_MENU           ,
    kbAltR        ,   VK_RMENU          ,
@@ -2840,7 +2923,7 @@ int ctx_kb2VK[] = {
    endCtx
 };
 
-int ctx_kb2VK2[] = {
+Handle ctx_kb2VK2[] = {
    kbBackspace   ,   VK_BACK           ,
    kbTab         ,   VK_TAB            ,
    kbEsc         ,   VK_ESCAPE         ,
@@ -2849,26 +2932,28 @@ int ctx_kb2VK2[] = {
    endCtx
 };
 
-int ctx_kb2VK3[] = {
+Handle ctx_kb2VK3[] = {
    kbAltL        ,   kbAltR            ,
    kbShiftL      ,   kbShiftR          ,
    kbCtrlL       ,   kbCtrlR           ,
    endCtx
 };
 
-
-typedef LRESULT (*ApiMessageSender)(HWND hwnd, UINT msg, WPARAM mp1, LPARAM mp2);
 Bool
 apc_message( Handle self, PEvent ev, Bool post)
 {
    ULONG msg;
    USHORT mp1s = 0;
-   ApiMessageSender sender = post ? (ApiMessageSender) PostMessage : (ApiMessageSender) SendMessage;
    objCheck false;
    switch ( ev-> cmd)
    {
        case cmPost:
-          sender(( HWND) var handle, WM_POSTAL, ( WPARAM) ev-> gen. H, ( LPARAM) ev-> gen. p);
+          if (post)
+             PostMessage(( HWND) var handle, WM_POSTAL, ( WPARAM) ev-> gen. H, ( LPARAM) ev-> gen. p);
+           else {
+             SendMessage(( HWND) var handle, WM_POSTAL, ( WPARAM) ev-> gen. H, ( LPARAM) ev-> gen. p);
+             exception_check_raise();
+          }
           break;
        case cmMouseMove:
           msg = WM_MOUSEMOVE;
@@ -3238,11 +3323,9 @@ apc_system_action( const char * params)
       }
       break;
    case 'w':
-      if ( strcmp( params, "wait.before.quit") == 0) {
-         waitBeforeQuit = 1;
-      } else if ( strncmp( params, "win32.DrawFocusRect ", 20) == 0) {
+      if ( strncmp( params, "win32.DrawFocusRect ", 20) == 0) {
          RECT r;
-         unsigned long win;
+         Handle win;
          Handle self;
          int i = sscanf( params + 20, "%lu %ld %ld %ld %ld", &win, &r.left, &r.bottom, &r.right, &r.top);
 
@@ -3254,42 +3337,18 @@ apc_system_action( const char * params)
          r. bottom = sys lastSize. y - r. bottom;
          r. top    = sys lastSize. y - r. top;
          DrawFocusRect( sys ps, &r);
-      } else if (strcmp( params, "win.printfocusedwindow") == 0) {
-         HWND h = GetFocus();
-         if ( h) {
-            char b[ 256];
-            DOLBUG( "%08x: %s", h, GetWindowText( h, b, 255) ? b : "NULL");
-         } else {
-            DOLBUG( "? No foc");
-         }
-      } else if (strncmp( params, "win32.WNetGetUser", 17) == 0) {
-         char connection[ 1024];
-         char user[ 1024], *c;
-         DWORD len = 1024;
-         int i = sscanf( params + 18, "%s", connection);
-         if ( i != 1) {
-            warn( "Bad parameters to sysaction win32.WNetGetUser");
-            return 0;
-         }
-         if ( WNetGetUser( connection, user, &len) != NO_ERROR)
-            return 0;
-         c = ( char *) malloc( strlen( user) + 1);
-         return c ? strcpy( c , user) : 0;
       } else if ( strncmp( params, "win32.SetVersion", 16) == 0) {
          const char * ver = params + 17;
          while ( *ver && ( *ver == ' '  || *ver == '\t')) ver++;
 
          if ( stricmp( ver, "NT") == 0) {
             guts. version = 0;
-            guts. is98 = 0;
          } else if (( stricmp( ver, "95") == 0) || ( stricmp( ver, "mustdie") == 0)) {
             guts. version = 0x80000095;
-            guts. is98 = 0;
          } else if ( stricmp( ver, "98") == 0) {
             guts. version = 0x80000098;
-            guts. is98 = 1;
          } else {
-            warn( "Are you nuts? Microsoft itself afraid of %s yet!", ver);
+            guts. version = 0x80000000;
          }
       } else if ( strncmp( params, "win32.ConsoleWindow", 19) == 0) {
          params += 19;
@@ -3305,7 +3364,7 @@ apc_system_action( const char * params)
 
          if ( strcmp( params, " exists") == 0) {
            char * p = ( char *) malloc(12);
-           if ( p) sprintf( p, "0x%08lx", ( unsigned long) guts. console);
+           if ( p) sprintf( p, "0x%08lx", ( Handle) guts. console);
            return p;
          } else
          if ( strcmp( params, " hide") == 0)     { ShowWindow( guts. console, SW_HIDE); } else
